% fc = computeFC(f, fsr, detune, Ti, loss)
% [r_u, r_d, rotDG] = computeFC(f, fsr, detune, Ti, loss)
%   filter cavity transfer matrix and loss matrix
%
% Arguments:
% f = frequency vector [Hz]
% fsr = c / (2 * L) = cavity free spectral range [Hz]
% detune = cavity detuning [Hz]
% Ti = input mirror power transmission
% loss = power loss, including end mirror transmission, in the cavity
%
% Return (1 arg):
% fc = structure with the following fields
%
%   f, fsr, detune, Ti, loss = input arguments
%   r_u, r_d = upper and lower sideband (up and down) reflectivity
%   rho_p, rho_m = average and delta magnitude
%   alpha_p, alpha_m = average and delta angle (radians)
%   tau2 = average power transmission
%   Tfc = 2-photon transfer matrix
%   .hf = structure with values from high-finesse approximation
%
%   rotDG = (180 / pi) * angle(r_u .* r_d) = rotation angle in degree
%
% Return (2 or 3 args):
% r_u, r_d = upper and lower sideband (up and down) reflectivity
% rotDG = (180 / pi) * angle(r_u .* r_d) = rotation angle in degree

function [r_u, r_d, rotDG] = computeFC(f, fsr, detune, Ti, loss)

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Direct Computation
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  % reflectivities
  re = 1;                         % assume no end mirror transmission
  ri = sqrt(1 - Ti);
  rr = ri * re * sqrt(1 - loss);  % include round-trip losses
  
  % Phases for positive and negative audio sidebands
  Phi_u = 2 * pi * (f - detune) / fsr;
  Phi_d = 2 * pi * (-f - detune) / fsr;
  
  ephi_u = exp(-1i * Phi_u);
  ephi_d = exp(-1i * Phi_d);
  
  % cavity gains
  g_u = 1 ./ ( 1 - rr * ephi_u);
  g_d = 1 ./ ( 1 - rr * ephi_d);
  
  % Reflectivity for vacuum flactuation entering the cavity from
  % the input mirror (check sign)
  r_u = ri - (rr / ri) * Ti * ephi_u .* g_u;
  r_d = ri - (rr / ri) * Ti * ephi_d .* g_d;
  
  % bail out here?
  if nargout == 2
    return
  end
  
  % rotation angle
  rotDG = (180 / pi) * unwrap(angle(r_u .* r_d)) / 2;
  rotDG_end = 180 * round(rotDG(end) / 180);
  rotDG = rotDG - rotDG_end;
  
  % bail out here?
  if nargout == 3
    return
  end
  
  %semilogx(w/(2*pi), angle([r_p ./ conj(r_m)]))
  %pause
  
  % Relfection matrix for vacuum fluctuations entering from the input
  % mirror in the A+, (a-)* basis
  Mr = make2x2TF(r_u, 0, 0, conj(r_d));
  
  % Apply matrix which changes from one-photon to two-photon
  Tfc = convert1p2p(Mr);
  
  % build struct
  fc.r_u = r_u;
  fc.r_d = r_d;
  fc.rho_p = (abs(r_u) + abs(r_d)) / 2;
  fc.rho_m = (abs(r_u) - abs(r_d)) / 2;
  fc.alpha_u = myarg(r_u);
  fc.alpha_d = myarg(r_d);
  fc.alpha_p = myarg(r_u .* r_d) / 2;
  fc.alpha_m = myarg(r_u ./ r_d) / 2;
  fc.tau2 = (abs(r_u).^2 + abs(r_d).^2) / 2;
  fc.Tfc = Tfc;
  fc.rotDG = rotDG;

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % High Finesse Approximation
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  % cavity half width
  gamma = fsr * (Ti + loss) / 2;
  eps = 2 * loss / (Ti + loss);
  
  % Phases for positive and negative audio sidebands
  w = 2 * pi * f;
  deltaW = 2 * pi * detune;
  xi_u = (w - deltaW) / gamma;
  xi_d = (-w - deltaW) / gamma;
  
  % reflectivities
  r_u = 1 - (2 - eps) ./ (1 + 1i * xi_u);
  r_d = 1 - (2 - eps) ./ (1 + 1i * xi_d);
  
  % cavity phases and magnitudes
  alpha_u = myatan2((2 - eps) * xi_u, -1 + eps + xi_u.^2);
  alpha_d = myatan2((2 - eps) * xi_d, -1 + eps + xi_d.^2);
%   alpha_u = arg(1i * (2 - eps) * xi_u - 1 + eps + xi_u.^2);
%   alpha_d = arg(1i * (2 - eps) * xi_d - 1 + eps + xi_d.^2);
  
%  alpha_p = (alpha_u + alpha_d) / 2;
  % small epsilon approximation
  alpha_p = atan2(((2 - eps) * gamma * deltaW), ...
    ((1 - eps) * gamma^2 - deltaW.^2 + w.^2));
  
  alpha_m = (alpha_u - alpha_d) / 2;
  
  rho_u = sqrt(1 - (2 - eps) * eps ./ (1 + xi_u.^2));
  rho_d = sqrt(1 - (2 - eps) * eps ./ (1 + xi_d.^2));
  
  rho_p = (rho_u + rho_d) / 2;
  rho_m = (rho_u - rho_d) / 2;
  
  % Relfection matrix for vacuum fluctuations
  % entering from the input mirror in the 2-photon basis

  % lossless FC matrix
  Rap = make2x2Rot(alpha_p);
  eam = exp(1i * alpha_m);
  Mr_lossless = getProdTF(make2x2TF(eam, 0, 0, eam), Rap);

  % mixing due to FC losses
  Mr_mix = make2x2TF(rho_p, 1i * rho_m, -1i * rho_m, rho_p);
  
  % total FC reflection matrix
  Tfc = getProdTF(Mr_lossless, Mr_mix);
  
  % build struct
  hf.r_u = r_u;
  hf.r_d = r_d;
  hf.rho_p = rho_p;
  hf.rho_m = rho_m;
  hf.alpha_u = alpha_u;
  hf.alpha_d = alpha_d;
  hf.alpha_p = alpha_p;
  hf.alpha_m = alpha_m;
  hf.tau2 = rho_p.^2 + rho_m.^2;
  hf.Tfc = Tfc;
  fc.hf = hf;
  
  % map to output name
  r_u = fc;
end

% th = myarg(z)
%   arg which tries to make things continuous (from high to low)
%
% Implementation:
%   th = flipud(unwrap(flipud(angle(z))));
function th = myarg(z)

  th = flipud(unwrap(flipud(angle(z))));
    
end

% th = myatan2(y, x)
%   atan2 which tries to make things continuous (from high to low)
%
% Implementation:
%   th = flipud(unwrap(flipud(atan2(y, x))));
function th = myatan2(y, x)

  th = flipud(unwrap(flipud(atan2(y, x))));
    
end
